/*
   Informixtcl.h ---

   This include file contains all information 

   This TCL-extension is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Written by Joachim Siebold <joachim.siebold@cityweb.de>
   based on DBD::Informix (Copyright J. Leffler and others)

   $Header: /home/cvs/informixtcl/InformixTcl.h,v 1.4 2000/04/04 16:22:35 achim Exp $
   $Log: InformixTcl.h,v $
   Revision 1.4  2000/04/04 16:22:35  achim
   Changed package name to informixTcl, introduced namespace informixTcl

   Revision 1.3  2000/03/19 12:11:24  achim
   Derive release number from configure.in

   Revision 1.2  2000/03/18 18:24:58  achim
   Introduced CVS Keywords

*/         

#ifndef _INFORMIX_SQL
#define _INFORMIX_SQL

#include "esqlc.h"
#include "InformixTclVersion.h"

enum Boolean
{
    False, True
};
typedef enum Boolean Boolean;      

/*
 *  Program Constants
 */

#define     STATEMENT_MAGIC             24+8+1959
#define     LOGON_MAGIC                 7+1+1960


#define     NAMESIZE 19                 /* 18 character name plus '\0' */
#define     DEFAULT_DATABASE            ".DEFAULT."

/* 
    Different states for a statement 
 */
enum State
{
    Unused, Prepared, Allocated, Described, Declared, Opened, Finished
};

typedef enum State       State;         /* Cursor/Statement states */

/* 
    INFORMIX Connect Options 
 */
#define     INFORMIX_CONNOPT_NONE       0
#define     INFORMIX_CONNOPT_AUTOCOMMIT 1

typedef long ErrNum;                    /* Informix Error Number */
typedef char Name[NAMESIZE];      

enum BlobLocn
{
    BLOB_DEFAULT, BLOB_IN_MEMORY, BLOB_IN_ANONFILE, BLOB_IN_NAMEFILE
};
typedef enum BlobLocn BlobLocn;   

/*
 * Program Data Types 
 */

typedef struct _LogonInfo {
    int                 magic;
    Tcl_Interp          *interp;            /* TCL Interpreter */
    Name                logonName;          /* Name of Handle */
    Name                nm_connection;      /* Connection Name */
    char                *database;          /* Name of Database */
    Boolean             is_connected;       /* Is connection open */
    Boolean             is_onlinedb;        /* Is OnLine Engine */
    Boolean             is_modeansi;        /* Is MODE ANSI Database */
    Boolean             is_loggeddb;        /* Has transaction log */
    Boolean             is_txactive;        /* Is inside transaction */
    Boolean             chop_blanks;        /* chop trailing blanks? */
    Boolean             has_autocommit;     /* Autocommit enabled */
    Sqlca               ix_sqlca;           /* Last SQLCA record for 
                                                connection */

} LogonInfo;    

typedef struct _StatementInfo {
    LogonInfo           *logonVars;
    int                  magic;
    Name                statementName;      /* Name of Handle */
    Name                nm_stmnt;           /* Name of prepared statement */
    Name                nm_obind;           /* Name of allocated descriptor */
    Name                nm_cursor;          /* Name of declared cursor */
    Name                nm_ibind;           /* Name of input (bind)descriptor */
    State               st_state;           /* State of statement */
    int                 st_type;            /* Type of statement */
    BlobLocn            blob_bind;          /* Blob Binding */
    int                 n_blobs;            /* Number of blobs for statement */
    int                 n_columns;          /* Number of output fields */
    int                 n_bound;            /* Number of input fields */
    int                 n_rows;             /* Number of rows processed */
    int                 n_input_params;     /* Number of Input Parameters */
    
}   StatementInfo;

typedef struct _ObjProcTable {
    char                *name;
    Tcl_ObjCmdProc      *proc;
} ObjProcTable;

typedef struct _InformixSysInfo {    
    Tcl_HashTable       logonHash;          /* Logon Handle Hash */
    Tcl_HashTable       statementHash;      /* Statement Handle Hash */
    LogonInfo           *current_connection;/* Handle of current connection */
    Boolean             multiconnections;   /* Supports multiple connections */
    int                 debug;              /* Debug Flag */
   
} InformixSysInfo;


/*
 * Message array
 */
#define     InformixMessages        "::informixTcl::infmsg"
#define     InformixVersion         "version"
#define     InformixRows            "rows"          /* Obsolete ? */
#define     InformixErrortxt        "errtxt"
#define     InformixErrornum        "errnum"
#define     InformixProdVersion     "prodversion"

#define InformixSetMsgString(interp,index,str) \
    Tcl_SetVar2 (interp, InformixMessages, index, str, 0)

#define InformixSetMsgStringAppend(interp,index,str) \
    Tcl_SetVar2 (interp, InformixMessages, index, str,  \
    TCL_APPEND_VALUE | TCL_LIST_ELEMENT)

#define InformixSetMsgInt(interp,index,val) { \
    char numBuf[80]; sprintf (numBuf, "%ld", val); \
    Tcl_SetVar2 (interp, InformixMessages, index, numBuf, 0); \
}

#define SetResult(interp,str) \
     (Tcl_SetStringObj (Tcl_GetObjResult (interp), (str), -1))
#define AppendResult(interp,str) \
     (Tcl_AppendToObj ( Tcl_GetObjResult( (interp) ), (str), -1))
#define SetIntResult(interp,val) \
     (Tcl_SetIntObj (Tcl_GetObjResult (interp), (val)))




/*
 * procedure prototypes
 */

InformixSysInfo *InformixTclGetSysinfo _ANSI_ARGS_(( ));
int InformixTclSqlInit _ANSI_ARGS_ ((Tcl_Interp *interp )); 
int InformixTclPrepareCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclCloseCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclReleaseCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclExecuteCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclFetchCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclCommitCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclRollbackCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclLogonCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclLogoffCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclGetInfoCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclSetCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclDisconnectAllCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclDosqlCmd _ANSI_ARGS_ ((ClientData dummy,
        Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[]));
int InformixTclStatementHandle _ANSI_ARGS_ (( Tcl_Interp *interp,
        char * statementName, StatementInfo ** statementVars));
int InformixTclNewStatementHandle _ANSI_ARGS_ ((Tcl_Interp *interp,
        LogonInfo *logonVars, StatementInfo **statementVars));
int InformixTclDeleteStatementHandle _ANSI_ARGS_ ((Tcl_Interp *interp,
        StatementInfo *statementVars));
int InformixTclCheckStatement _ANSI_ARGS_((Tcl_Interp *interp,
        StatementInfo *statementVars));
int InformixTclLogonHandle _ANSI_ARGS_ ((Tcl_Interp *interp,
        char *logonName, LogonInfo **logonVars));
int InformixTclNewLogonHandle _ANSI_ARGS_ ((Tcl_Interp *interp,
        LogonInfo **logonVars));
int InformixTclDeleteLogonHandle _ANSI_ARGS_ ((Tcl_Interp *interp,
        LogonInfo *logonVars));
int InformixTclCheckLogon _ANSI_ARGS_((Tcl_Interp *interp,
        LogonInfo *logon));
char * stalloc _ANSI_ARGS_ (( char * s));


int InformixApiLogon _ANSI_ARGS_ ((Tcl_Interp *interp, LogonInfo
   *logonVars, char *name, char *user, char *pass, int conn_opt));
int InformixApiLogoff _ANSI_ARGS_ ((Tcl_Interp *interp, LogonInfo
    *logonVars));
void InformixApiDebug _ANSI_ARGS_ ((int n, char * fmt, char *arg));
void InformixApiDebugl _ANSI_ARGS_ ((int n, char *fmt, long arg));
void InformixApiSetError _ANSI_ARGS_ ((Tcl_Interp *interp, ErrNum rc));
void InformixApiSetError2 _ANSI_ARGS_ ((LogonInfo *logonVars, char *msg));
void InformixApiEnter _ANSI_ARGS_ ((char *func));
void InformixApiExit _ANSI_ARGS_ ((char *func));
void InformixApiSaveSqlca _ANSI_ARGS_ ((LogonInfo *logonVars));
void InformixApiSqlCode _ANSI_ARGS_ ((LogonInfo *logonVars));
int  InformixApiSetConnection _ANSI_ARGS_ ((LogonInfo *logonVars));
int InformixApiBegin _ANSI_ARGS_ ((Tcl_Interp *interp, LogonInfo *logonVars));
int InformixApiCommit _ANSI_ARGS_ ((Tcl_Interp *interp, LogonInfo *logonVars));
int InformixApiRollback _ANSI_ARGS_ ((Tcl_Interp *interp, LogonInfo *logonVars));
int InformixApiPrepare _ANSI_ARGS_ ((Tcl_Interp *interp, StatementInfo *statementVars, char *cmd));
int InformixApiClose _ANSI_ARGS_ ((Tcl_Interp *interp, StatementInfo *statementVars));
int InformixApiRelease _ANSI_ARGS_ ((Tcl_Interp *interp, StatementInfo *statementVars));
int InformixApiExecute _ANSI_ARGS_ ((Tcl_Interp *interp, StatementInfo *statementVars, int objc, Tcl_Obj *CONST objv[]));
int InformixApiFetch _ANSI_ARGS_ ((Tcl_Interp *interp, StatementInfo *statementVars, int objc, Tcl_Obj *CONST objv[]));
int InformixApiDosql _ANSI_ARGS_ ((Tcl_Interp *interp, LogonInfo *logonVars, char * statement));
int InformixApiGetAttrib _ANSI_ARGS_ ((Tcl_Interp *interp, StatementInfo *statementVars, int itype));
int InformixApiDisconnectAll _ANSI_ARGS_ ((Tcl_Interp *interp));

int InformixIntLogon _ANSI_ARGS_ ((LogonInfo *logonVars, char *name, char* user, char *pass));
int InformixIntBegin _ANSI_ARGS_ ((LogonInfo *logonVars));
int InformixIntCommit _ANSI_ARGS_ ((LogonInfo *logonVars));
int InformixIntRollback _ANSI_ARGS_ ((LogonInfo *logonVars));
int InformixIntLogoff _ANSI_ARGS_ ((LogonInfo *logonVars));
int InformixIntPrepare _ANSI_ARGS_ ((StatementInfo *statementVars, char *cmd));
int InformixIntDelStatement _ANSI_ARGS_ ((StatementInfo *statementVars));
int InformixIntSetBindNum _ANSI_ARGS_ ((StatementInfo *statementVars, int items));

int InformixIntClose _ANSI_ARGS_ ((StatementInfo *statementVars));
int InformixIntPreparse _ANSI_ARGS_ ((StatementInfo *statementVars, char *cmd));
int InformixIntFetch _ANSI_ARGS_ ((StatementInfo *statementVars,int objc, Tcl_Obj *CONST objv[], Tcl_Obj *reslist));
int InformixIntOpen _ANSI_ARGS_ ((StatementInfo *statementVars));
int InformixIntBlobs _ANSI_ARGS_ ((StatementInfo *statementVars));
int InformixIntDeclare _ANSI_ARGS_ ((StatementInfo *statementVars));
int InformixIntRelease _ANSI_ARGS_ ((StatementInfo *statementVars));
int InformixIntExec _ANSI_ARGS_ ((StatementInfo *statementVars));
void InformixIntSetDbType _ANSI_ARGS_ ((LogonInfo *logonVars));
int InformixIntBind _ANSI_ARGS_ ((StatementInfo *statementVars, int count, Tcl_Obj *obj));
int InformixIntGetAttrib _ANSI_ARGS_ ((StatementInfo *statementVars, int ind, int itype, long *retlong,char *retstring));
 
char *decgen _ANSI_ARGS_ ((dec_t *val, int plus));
int count_blobs _ANSI_ARGS_ ((char *descname, int ncols));

int InformixIntOpenDatabase _ANSI_ARGS_ ((char *dbase));
void InformixIntCloseDatabase _ANSI_ARGS_ ((char *dbase));
void InformixIntSetConnection _ANSI_ARGS_ ((char *conn));
int InformixIntConnect _ANSI_ARGS_ ((char *connection, char *dbase, char *user, char *pass));
void InformixIntDisconnect _ANSI_ARGS_ ((char *conn)); 
int InformixIntDosql _ANSI_ARGS_ ((LogonInfo *logonVars, char *cmd));


#endif /* _INFORMIX_SQL */
